/*
 * (./) XML.java 05/12/17 by cs for "MMM" project
 * (cc) some right reserved
 *
 * todo:
 *	+- use DOM to generate XML document
 *	+- implement an XML error handler
 */


package mmm.io;

import java.io.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.*;
import org.xml.sax.SAXException;


/* a simple class to read and write XML content
 */
public class XML {

	public final static String CHARSET	= "utf-8";
	public final static float VERSION	= 1.0f;
	
	private OutputStreamWriter out	= null;
	private Document doc			= null;
	private Element root			= null;
	
	/* open an existing XML document with DOM
	 * @arg File, the XML file
	 */
	public void open( File f ) throws Exception {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setIgnoringComments( true );
			doc  = factory.newDocumentBuilder().parse( f );
			root = doc.getDocumentElement();
		} catch ( SAXException e ) {
			// A parsing error occurred; the xml input is not valid
			throw new Exception( "parsing XML error: "+e.getMessage() );
		}
		catch( ParserConfigurationException e ) {;} 
		catch( IOException e ) {;}
	}
	
	/* get the root attributs
	 * @arg String, the attributs name to be returned
	 */
	public String getRootAttribut( String name ) {
		if ( doc==null ) return "";
		//Element root = doc.getDocumentElement();
		return root.getAttribute( name );
	}

	/* return the number of nodes in the root element
	 */
	public int getLength() {
		if ( doc==null ) return 0;
		//Element root = doc.getDocumentElement();
		return root.getChildNodes().getLength();
	}
	
	/* get the require node ID attribut
	 * @arg String, the node ID
	 * @arg String, the attributs name to be returned
	 */
	//public String getIDAttribut( String id, String name ) {
	public String getIDAttribut( int i, String name ) {
		/*try {
			Element node = doc.getElementById( id );
			System.out.println( node );
			return node.getAttribute( name );
		}
		catch( NullPointerException e ) {;} // ID not found
		return "";*/
			
		NodeList childs	= root.getChildNodes();
		Node node		= childs.item(i).getAttributes().getNamedItem(name);
		
		return node.getNodeValue();
	}
	
	/* get the require node ID attribut
	 * @arg int, the node index
	 * @arg String, the tag name
	 */
	public String getTagValue( int i, String name ) {
		
		NodeList tags	= root.getChildNodes().item(i).getChildNodes();
		
		for(int j=0; j<tags.getLength(); j++) {
			Node tag = tags.item(j);
			if ( tag.getNodeType()==Node.ELEMENT_NODE &&
				 tag.getNodeName().equals(name) ) 
				return tag.getChildNodes().item(0).getNodeValue();
		}
		return null;
	}
	
	/* get the require node ID attribut
	 * @arg int, the node index
	 * @arg String[], the tag content as String[]
	 */
	public String[] getTagAsObject( int i, String name ) {
		
		NodeList tags	= root.getChildNodes().item(i).getChildNodes();
		
		for(int j=0; j<tags.getLength(); j++) {
			Node tag = tags.item(j);
			if ( tag.getNodeType()==Node.ELEMENT_NODE &&
				 tag.getNodeName().equals(name) ) {
				Node node = tag.getChildNodes().item(0);
				if ( node!=null ) return node.getNodeValue().split(",");
			}
		}
		return new String[0];
	}
	
	/* create a new XML document
	 * @arg File, the target file
	 */
	public void create( File f ) {
		
		try {
			OutputStream fs = new FileOutputStream( f );
			OutputStream bs = new BufferedOutputStream( fs );
			
			out = new OutputStreamWriter( bs, CHARSET );
			writeln( "<?xml version=\"" + VERSION +
				     "\" encoding=\""+ CHARSET + "\"?>" );
		}
		catch( FileNotFoundException e ) {;}
		catch( UnsupportedEncodingException e ) {;}
	}
	
	/* write
	 * implment auto-tabulation
	 */
	public void writeln( String s ) { write( s+"\n" ); }
	public void write( String s ) {
		try {
			if ( out!=null ) out.write( s );
		}
		catch( IOException e ) { System.err.println( e.getMessage() ); }
	}
	
	/* close current handler
	 */
	public void close() {
		try {
			if ( out!=null ) {
				out.flush();
				out.close();
				out = null;
			}
		}
		catch( IOException e ) { System.err.println( e.getMessage() ); }
	}
}
