/*
 * (./) Editor.java 05/11/03 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.TableModel;

import mmm.*;
import mmm.event.WindowManager;
import mmm.media.*;
import mmm.io.XMLFilter;


/* A single frame containing miscellaneous tools to manage the 
 * project content.
 */
public class Editor extends Project implements	ActionListener, KeyListener, 
	TableModelListener, MouseInputListener, Runnable {

	// ui
	private JFrame frame			= null;
	private FullScreenWindow fswin	= null;
	private ExpandPane videoPane	= new ExpandPane( "Video :" );
	private ExpandPane soundPane	= new ExpandPane( "Sound :" );
	private Toolbar toolbar			= new Toolbar();
	private StatusBar statusbar		= new StatusBar();
	private TrackManager manager	= new TrackManager();
	private TrackGraphic graphic	= new TrackGraphic();
	private Timeline timeline		= new Timeline();
	
	// menu labels
	private final String MENU_FILE				= "File";
    private final String MENU_NEW_PROJECT		= "New Project";
    private final String MENU_OPEN_PROJECT		= "Open Project";
    private final String MENU_OPEN_QUICKLY		= "Open Quickly";
    private final String MENU_SAVE_PROJECT		= "Save";
    private final String MENU_SAVEAS_PROJECT	= "Save As...";
    private final String MENU_CLOSE_PROJECT		= "Close Project";
	private final String MENU_EDIT				= "Edit";
	private final String MENU_CUT				= "Cut";
	private final String MENU_COPY				= "Copy";
	private final String MENU_COPY_ALL			= "Copy All";
	private final String MENU_PASTE				= "Paste";
	private final String MENU_DELETE			= "Delete";
	private final String MENU_PROJECT			= "Project";
	private final String MENU_REMOTE_PLAY		= "Play...";
	private final String MENU_REMOTE_REWIND		= "Rewind";
	private final String MENU_REMOTE_STOP		= "Stop";
	private final String MENU_MANAGE_TRACK		= "Manage Tracks...";
	private final String MENU_ADD_TRACK			= "Add";
	private final String MENU_INSERT_TRACK		= "Insert";
	private final String MENU_DUPLICATE_TRACK	= "Duplicate";
	private final String MENU_REMOVE_TRACK		= "Remove";
	private final String MENU_MOVEDOWN_TRACK	= "Move Down";
	private final String MENU_MOVEUP_TRACK		= "Move Ups";
    private final String MENU_VIDEO				= "Video";
    private final String MENU_IMPORT_VIDEO		= "Import Video...";
    private final String MENU_REMOVE_VIDEO		= "Remove Video";
    private final String MENU_FULL_SCREEN		= "Full Screen";
	
	// font
	protected static final Font FONT_PLAIN	= new Font( null, Font.PLAIN, 11 );
	protected static final Font FONT_BOLD	= new Font( null, Font.BOLD, 11 );
	protected static final Font FONT_ITALIC	= new Font( null, Font.ITALIC, 11 );
	
	// graphics border
	protected static final int GRAPHIC_RIGHT_MARGIN = 21;
	
	// listener
	private WindowManager winMgr = null;
	
	// media
	private Video video	= new Video();
	
	// is in full screen mode
	private boolean isFullScreen = false;
	
	// this thread
	private Thread thread = null;
	
	// the selected track
	private Track track = null;

	// the current signal index and entry type.
	// Require by mouse events to compare this value
	private int ioType	= Track.TYPE_OUT;
	private int ioIndex	= 0 ;
	
	// use to know if this editor is modified or not
	private boolean modified = false;
	
	// default directories
	private String DIR_PROJECTS	= "pièces";
	private String DIR_MEDIAS	= "medias";
	
	// store temporary signals values
	double[] clipboard = new double[0];
	
	
	/* create a new Editor window
	 * @arg WindowListener, the frame window listener
	 * @arg [ File, a valid file path to launch a existent project
	 *		or nothing to create a project by default ]
	 */
	public Editor( WindowManager wm ) {
		init( wm );
	}
	public Editor( WindowManager wm, File file ) {
		
		super(file);
		init( wm );
		
		// load media if needed
		if ( getVideo()!=null ) {
			if ( video.load( "file:"+getVideo()) ) {;
				videoPane.setText(  "video: "+getVideo().getName() );
				videoPane.setEnabled( true );
				videoPane.setExpanded( true );
			}
		}
	}
	
	
	/* initialize the interface
	 */
	private void init( WindowManager wm ) {
		
		GraphicsEnvironment env;
		
		frame	= new JFrame( super.getName() );
		fswin	= new FullScreenWindow( frame );
		env		= GraphicsEnvironment.getLocalGraphicsEnvironment();
		
		// check dependency
		checkDependency();
		
		// add the menu
		setMenu();
		
		// set the sound and video panels
		JPanel mediapane = new JPanel();
		videoPane.add( video );
		videoPane.setEnabled( false );
		soundPane.setEnabled( false );
		soundPane.setText( "Sound : not implemented yet ...");
		
		mediapane.setLayout( new BoxLayout(mediapane, BoxLayout.Y_AXIS) );
		mediapane.add( videoPane );
		mediapane.add( new JSeparator() );
		mediapane.add( soundPane );
		mediapane.add( new JSeparator() );
		
		// set the main panels : 
		// status bar, tools bar, track list & graphics...
		JPanel mainpane			= new JPanel();
		JSplitPane splitpane	= new JSplitPane();
		JPanel barpane			= new JPanel();
		JScrollPane gcpane		= new JScrollPane( graphic );
		
		barpane.setLayout( new BoxLayout(barpane, BoxLayout.Y_AXIS) );
		barpane.add( toolbar );
		barpane.add( statusbar );
		
		graphic.setOwner( gcpane );
		gcpane.setColumnHeaderView( timeline );
		gcpane.getVerticalScrollBar().setUnitIncrement(TrackGraphic.SIZE_MEDIUM);
		
		splitpane.setLeftComponent( manager );
		splitpane.setRightComponent( gcpane );
		splitpane.setOneTouchExpandable( true );
		
		mainpane.setLayout( new BorderLayout(3, 3) );
		mainpane.add( barpane, BorderLayout.NORTH );
		mainpane.add( splitpane, BorderLayout.CENTER );
		
		// init components
		setComponents();
		
		// register to the window manager
		winMgr = wm;
		winMgr.add( this, frame );
		
		// init frame
		frame.setBounds( 0, 0, 640, 480 );
		frame.setBounds( env.getMaximumWindowBounds() );
		frame.getContentPane().add( mediapane, BorderLayout.NORTH );
		frame.getContentPane().add( mainpane );
		frame.setDefaultCloseOperation( frame.DO_NOTHING_ON_CLOSE );
		frame.setDefaultLookAndFeelDecorated( false );
		frame.addWindowListener( wm );
		frame.addKeyListener( this );
		frame.setVisible( true );
		
	}
	
	/* menu setup
	 */
	private void setMenu() {
		
		MenuBar mbar	= new MenuBar();
		Menu mfile		= new Menu( MENU_FILE );
		Menu mopen		= new Menu( MENU_OPEN_QUICKLY );
		Menu medit		= new Menu( MENU_EDIT );
		Menu mproject	= new Menu( MENU_PROJECT );
		Menu mtrack		= new Menu( MENU_MANAGE_TRACK );
		Menu mvideo		= new Menu( MENU_VIDEO );
		
		MenuItem minp	= new MenuItem( MENU_NEW_PROJECT, 
										new MenuShortcut(KeyEvent.VK_N));
		MenuItem miop	= new MenuItem( MENU_OPEN_PROJECT, 
										new MenuShortcut(KeyEvent.VK_O));
		MenuItem misp	= new MenuItem( MENU_SAVE_PROJECT, 
										new MenuShortcut(KeyEvent.VK_S));
		MenuItem misa	= new MenuItem( MENU_SAVEAS_PROJECT );
		MenuItem micp	= new MenuItem( MENU_CLOSE_PROJECT, 
										new MenuShortcut(KeyEvent.VK_W));
		MenuItem micx	= new MenuItem( MENU_CUT, 
										new MenuShortcut(KeyEvent.VK_X));
		MenuItem micc	= new MenuItem( MENU_COPY, 
										new MenuShortcut(KeyEvent.VK_C));
		MenuItem mica	= new MenuItem( MENU_COPY_ALL, 
										new MenuShortcut(KeyEvent.VK_C, true));
		MenuItem mipv	= new MenuItem( MENU_PASTE, 
										new MenuShortcut(KeyEvent.VK_V));
		MenuItem mied	= new MenuItem( MENU_DELETE );
		MenuItem mirp	= new MenuItem( MENU_REMOTE_PLAY );
		MenuItem mirr	= new MenuItem( MENU_REMOTE_REWIND );
		MenuItem mirs	= new MenuItem( MENU_REMOTE_STOP );
		MenuItem miat	= new MenuItem( MENU_ADD_TRACK );
		MenuItem miit	= new MenuItem( MENU_INSERT_TRACK );
		MenuItem midt	= new MenuItem( MENU_DUPLICATE_TRACK );
		MenuItem mirt	= new MenuItem( MENU_REMOVE_TRACK );
		MenuItem mimd	= new MenuItem( MENU_MOVEDOWN_TRACK,
										new MenuShortcut(KeyEvent.VK_UP, true));
		MenuItem mimu	= new MenuItem( MENU_MOVEUP_TRACK,
										new MenuShortcut(KeyEvent.VK_DOWN, true));
		MenuItem miiv	= new MenuItem( MENU_IMPORT_VIDEO );
		MenuItem mirv	= new MenuItem( MENU_REMOVE_VIDEO );
		MenuItem mifs	= new MenuItem( MENU_FULL_SCREEN, 
										new MenuShortcut(KeyEvent.VK_F));
		
		
		// list projects
		mopen.addActionListener( this );
		String[] files = (new File(DIR_PROJECTS)).list( new XMLFilter() );
		for( int i=0; i<files.length; i++ )
			mopen.add( new MenuItem(files[i]) );
			
		// menu file
		minp.addActionListener( this );
		miop.addActionListener( this );
		misp.addActionListener( this );
		misa.addActionListener( this );
		micp.addActionListener( this );
		mfile.add( minp );
		mfile.add( miop );
		mfile.add( mopen );
		mfile.addSeparator();
		mfile.add( misp );
		mfile.add( misa );
		mfile.add( micp );
		
		// menu edit
		micx.addActionListener( this );
		micc.addActionListener( this );
		mica.addActionListener( this );
		mipv.addActionListener( this );
		mied.addActionListener( this );
		medit.add( micx );
		medit.add( micc );
		medit.add( mica );
		medit.add( mipv );
		medit.addSeparator();
		medit.add( mied );
		
		// menu project
		// remote
		mirp.addActionListener( this );
		mirr.addActionListener( this );
		mirs.addActionListener( this );
		mproject.add( mirp );
		mproject.add( mirr );
		mproject.add( mirs );
		mproject.addSeparator();
		// submenu track
		miat.addActionListener( this );
		miit.addActionListener( this );
		midt.addActionListener( this );
		mirt.addActionListener( this );
		mimd.addActionListener( this );
		mimu.addActionListener( this );
		mtrack.add( miat );
		mtrack.add( miit );
		mtrack.add( midt );
		mtrack.add( mirt );
		mtrack.addSeparator();
		mtrack.add( mimd );
		mtrack.add( mimu );
		mproject.add( mtrack ); mtrack.setEnabled( false );
		// submenu video
		miiv.addActionListener( this );
		mirv.addActionListener( this );
		mvideo.add( miiv );
		mvideo.add( mirv );
		mproject.add( mvideo );
		// options
		mifs.addActionListener( this );
		mproject.addSeparator();
		mproject.add( mifs );
		
		// set menu bar
		mbar.add( mfile );
		mbar.add( medit );
		mbar.add( mproject ); 
		frame.setMenuBar( mbar );
    }
	
	/* init the miscellaneous properties required by the components
	 */
	private void setComponents() {
		
		// set toolbar data
		toolbar.setPosition( 0. );
		toolbar.setDuration( getDuration() );
		toolbar.setStart( 0. );
		toolbar.setStop( 0. );
		toolbar.setZoom( 1. );
			
		// set listener
		toolbar.setActionListener( this );
		manager.setTableModelListener( this );
		timeline.setMouseListener( this );
		graphic.setMouseListener( this );
		
		// set graphics
		int len = (int)getDuration();
		timeline.setLength( len );
		graphic.setWidth( len );
		
		// list tracks
		for( int i=0; i<getTrackCount(); i++ )
			manager.add( (getTrack(i)).toObject() );
		setModified( false ); // <-- due to tableCHanged() actions
		
	}
	
	/* set up the graphics size and the project length
	 */
	private void setDuration() {
		
		double len = toolbar.getDuration() ;
		
		timeline.setLength( (int)len );
		graphic.setWidth( (int)len );
		setDuration( len );
		
		//  update tracks
		for(int i=0; i<getTrackCount(); i++) {
			Track t = getTrack(i);
			t.setLength( len );
			graphic.update( t );
		}
		
		// updates graphics
		graphic.setSize();
		graphic.repaint();
		timeline.repaint();
		setModified( true );
		requestFocus();
	}
	
	/* set the cursor position
	 */
	public void setPosition() {
		
		int x = (int)toolbar.getPosition();
		
		timeline.setCursor( x );
		graphic.setCursor( x );
	}
	
	/* set the current track's signal in/out position
	 * @arg int, which output properties to be set 1=in 0=out
	 */
	public void setOutput( int which ) {
		
		if ( track==null ) return;
			
		// update signal position
		track.setSelectionType( which );
		track.set( which==Track.TYPE_OUT?toolbar.getStop():toolbar.getStart() );
		
		// update graphic
		//graphic.setSelection( track.index, track.getIn(), track.getOut() );
		graphic.update( track );
		graphic.repaint();
	}
	
	/* copy the selected signal
	 */
	public void copy() {
		if ( track!=null )
			clipboard = new double[]{ track.getIn(), track.getOut() };
	}
	
	/* copy all selected track's signals
	 */
	public void copyAll() {
		if ( track!=null ) clipboard = track.getOutputs();
	}
	
	/* paste the copied signals to the selected tracks
	 */
	//public void paste( boolean replace ) {
	public void paste() {
		if ( track==null || clipboard.length==0 ) return;
		
		// add entries
		for( int i=0; i<clipboard.length; i+=2 )
			track.add( clipboard[i], clipboard[i+1] );
		
		// update UI
		graphic.update( track );
		graphic.repaint();
		setModified( true );
	}
	
	
	/* remove the selected signals
	 */
	public void delete( boolean modify ) {
		if ( track==null ) return;
		
		if ( track.remove() ) {
			graphic.update( track );
			graphic.repaint();
			if ( modify ) setModified( true );
		}
	}
	
	/* set the track's graphic height
	 */
	private void setGraphicHeight() {
		
		int value = toolbar.getGraphicSize();
		
		// update graphics
		graphic.setHeight( value );
		for( int i=0; i<getTrackCount(); i++ ) graphic.update( getTrack(i) );
		graphic.repaint();
		
		requestFocus();
	}
	
	/* select/deselect the current track signal
	 * and enable/disable output fields
	 * @arg boolean, the require status
	 */
	public void setOutputEnabled( boolean val ) {
		
		if ( track==null ) return;
		
		// fields
		toolbar.setStart( val ? track.getIn() : 0. );
		toolbar.setStop( val ? track.getOut() : 0. );
		toolbar.setOutputFieldsEnabled( val );
	}
	
	/* Key actions performed
	 */
	public void keyPressed( KeyEvent e )	{;}
	public void	keyTyped( KeyEvent e )		{;} 
	public void	keyReleased( KeyEvent e )	{
		switch( e.getKeyCode() ) {
			
			// play/pause
			case KeyEvent.VK_SPACE:
				if ( isRunning() ) stop();
				else start();
				break;
				
			// remove selected signals
			case KeyEvent.VK_DELETE:
			case KeyEvent.VK_BACK_SPACE:
				delete( true );
				break;
				
			// exit fullscreen
			case KeyEvent.VK_ESCAPE:
				if ( isFullScreen ) setFullScreen( false );
		}
	}
	
	/* GUI actions performed
	 */
    public void actionPerformed( ActionEvent e ) {
		
		String cmd = e.getActionCommand();
		
		// menu action performed
		if ( cmd.equals(MENU_NEW_PROJECT) )		new Editor( winMgr );
		if ( cmd.equals(MENU_OPEN_PROJECT) )	open();
		if ( cmd.equals(MENU_SAVE_PROJECT) )	save();
		if ( cmd.equals(MENU_SAVEAS_PROJECT) )	saveAs();
		if ( cmd.equals(MENU_CLOSE_PROJECT) )	close();
		if ( cmd.equals(MENU_CUT) )				delete( true );
		if ( cmd.equals(MENU_COPY) )			copy();
		if ( cmd.equals(MENU_COPY_ALL) )		copyAll();
		if ( cmd.equals(MENU_PASTE) )			paste();
		if ( cmd.equals(MENU_DELETE) )			delete( true );
		if ( cmd.equals(MENU_REMOTE_PLAY) )		start();
		if ( cmd.equals(MENU_REMOTE_REWIND) )	rewind();
		if ( cmd.equals(MENU_REMOTE_STOP) )		stop();
		if ( cmd.equals(MENU_ADD_TRACK) )		manager.add();
		if ( cmd.equals(MENU_DUPLICATE_TRACK) ) manager.duplicate();
		if ( cmd.equals(MENU_INSERT_TRACK) )	manager.insert();
		if ( cmd.equals(MENU_REMOVE_TRACK) )	manager.remove();
		if ( cmd.equals(MENU_MOVEDOWN_TRACK) )	manager.moveDown();
		if ( cmd.equals(MENU_MOVEUP_TRACK) )	manager.moveUp();
		if ( cmd.equals(MENU_IMPORT_VIDEO) )	loadVideo();
		if ( cmd.equals(MENU_REMOVE_VIDEO) )	removeVideo();
		if ( cmd.equals(MENU_FULL_SCREEN) )		setFullScreen( !isFullScreen );
		
		// toolbar actions performed
		if ( cmd.equals(Toolbar.NEW_POSITION_VALUE) )	setPosition();
		if ( cmd.equals(Toolbar.NEW_DURATION_VALUE) )	setDuration();
		if ( cmd.equals(Toolbar.NEW_START_VALUE) )		setOutput( Track.TYPE_IN );
		if ( cmd.equals(Toolbar.NEW_STOP_VALUE) )		setOutput( Track.TYPE_OUT );
		//if ( cmd.equals(Toolbar.NEW_ZOOM_VALUE) ) setZoom();
		if ( cmd.equals(Toolbar.NEW_SIZE_VALUE) )		setGraphicHeight();
		
		// menu open quicktly action
		// lauch reuired project
		if ( cmd.endsWith(".xml") ) open( DIR_PROJECTS+"/"+cmd );
    }
	
	/* listen for changes in the tracks manager
	 * and perform the appropriate action
	 */
	public void tableChanged( TableModelEvent e ) {
		
		int row = e.getFirstRow();
        int col = e.getColumn();
		
		TableModel tm = (TableModel) e.getSource();
		
		// parse events
		switch ( e.getType() ) {
			
			// insert : append or insert a new tracks
			case TableModelEvent.INSERT :
				Track track;
				double[] output = new double[0];
				// TMP COPY OUTPUTS (for duplicate or append method only)
				if ( row<getTrackCount() ) {
					track  = getTrack(row);
					output = track.getOutputs();
				}
					
				// create the new track reference
				setTrack( row,
						  ((Boolean)tm.getValueAt(row,1)).booleanValue(),
						  ((Boolean)tm.getValueAt(row,2)).booleanValue(),
						  (Color) tm.getValueAt(row,3),
						  (String) tm.getValueAt(row,4)
						  );
				
				track = getTrack(row);
				for( int j=0; j<output.length; j+=2)
					track.add( output[j], output[j+1] );
				
				// update graphic
				// get the Track from the references list to be sure 
				// that we got the real value
				graphic.insert( track );
				break;
				
			// delete the selected track
			case TableModelEvent.DELETE :
				// forget reference
				removeTrack( row );
				// update graphic
				graphic.remove( row );
				break;
				
			// update
			case TableModelEvent.UPDATE :
                if ( row!=TableModelEvent.HEADER_ROW ) { 
					
					// move (swicth) tracks
					if ( col==TableModelEvent.ALL_COLUMNS ) {
						for( int i=row; i<=e.getLastRow(); i++ ) {
							setTrack(
								i,
								((Boolean)tm.getValueAt(i,1)).booleanValue(),
								((Boolean)tm.getValueAt(i,2)).booleanValue(),
								(Color)tm.getValueAt(i,3),
								(String)tm.getValueAt(i,4)
							);
							graphic.update( getTrack(i) );
						}
					}
					// modify the chnaged track value
					else {
						
						Track t	 = getTrack( row );
						Object o = tm.getValueAt( row, col );
						
						// get the changed track value, and set it
						switch ( col ) {
							// the track status
							case 1 :
								t.setEnabled(((Boolean)o).booleanValue());
								break;
							// the track visibility
							case 2 :
								t.setVisible(((Boolean)o).booleanValue());
								break;
							// the track color
							case 3 :
								t.setColor( (Color)o );
								break;
							// the track name
							case 4 :
								t.setName( (String)o );
								break;
						}
						
						// update graphic
						graphic.update( t );
						requestFocus();
					}
                }
                break;
		}
		
		// repaint graphic
		graphic.setSize();
		graphic.repaint();
		// change status
		setModified( true );
    }
	
	/* mouse events : 
	 * call the mouseActionPerformed() method for more flexibility
	 */
	public void mouseClicked( MouseEvent e )	{ mouseActionPerformed(e); } 
	public void mouseDragged( MouseEvent e )	{ mouseActionPerformed(e); } 
	public void mouseEntered( MouseEvent e )	{ mouseActionPerformed(e); } 
	public void mouseExited( MouseEvent e )		{ mouseActionPerformed(e); } 
	public void mouseMoved( MouseEvent e )		{ mouseActionPerformed(e); }
	public void mousePressed( MouseEvent e )	{ mouseActionPerformed(e); }
	public void mouseReleased( MouseEvent e )	{ mouseActionPerformed(e); }
	
	/* perform the appropriate mouse event action
	 * @arg MouseEvent, the current mouse event
	 */
	public  void mouseActionPerformed( MouseEvent e ) {
		
		int x		= e.getX();
		int y		= e.getY();
		
		// get the component source
		boolean fromGC = e.getSource() instanceof TrackGraphic;
		// set and limit the current time position
		double time = Math.max( Math.min(getDuration(),(double)x), 0 );
		
		// parse mouse event
		switch( e.getID() ) {
			
			// on mouse enter :
			// request focus to performe key events
			case MouseEvent.MOUSE_ENTERED :
				requestFocus();
				break;
				
			// on mouse release :
			// verify if the in/out values are different.
			// if not, clear the current signal 
			case MouseEvent.MOUSE_RELEASED :
				if ( track!=null && track.getIn()==track.getOut() )
					delete( false );
				break;
			
			// on mouse click :
			// set the cursor and the video position
			// select/deselect outputs
			case MouseEvent.MOUSE_CLICKED :
				
				// set the new position
				video.setTime( time );
				if ( isRunning() ) {
					setChrono( time );
					break; // do nothing more
				}
				
				// deselect the selection,
				// if no other selection have been already set
				if ( fromGC && track!=null &&
					 track.getSelectionType()==ioType &&
					 track.getSelectionIndex()==ioIndex )
				{
					track.deselect();
					graphic.update( track );
					graphic.repaint();
				}
				break;
				
			// if the mouse is dragged : 
			// change the output in/out value
			case MouseEvent.MOUSE_DRAGGED:
				if ( isRunning() || !fromGC ) return; // do nothing
				
				if ( track!=null ) {
					track.set( time );
					setOutputEnabled( true );
					toolbar.setPosition( time );
					timeline.setCursor( x );
					graphic.update( track );
					graphic.setCursor( x );
					setModified( true );
				}
				break;
				
			// if the mouse move : display the current time value
			// and move the timeline cursor
			case MouseEvent.MOUSE_MOVED:
				if ( isRunning() ) return; // do nothing
				
				toolbar.setPosition( x );
				timeline.setCursor( x );
				graphic.setCursor( x );
				break;
				
			// on mouse exit event :
			// place the cursor to the appropriate coordinates
			case MouseEvent.MOUSE_EXITED :
				if ( isRunning() ) return; // do nothing
				
				toolbar.setPosition( time );
				timeline.setCursor( (int)time );
				graphic.setCursor( (int)time );
				break;
				
			// if the mouse is pressed :
			// select/deselect or add a new output
			// update GUI
			case MouseEvent.MOUSE_PRESSED:
				if ( isRunning() || !fromGC ) return; // do nothing
				
				int i = graphic.getTrackIndex( x, y );
				
				if ( i==-1 ) { 
					if ( track!=null) { // deselect
						manager.deselectAll();
						setOutputEnabled( false );
						track.deselect();
						track = null;
						graphic.selectTrack( -1 );
						graphic.repaint();
					}
					break;
				}
		
				// select the clicked track
				// and deselect the previous track
				boolean change = ( track!=null && i!=track.index );
				if ( track==null || change ) {
					if ( change ) {
						track.deselect();
						graphic.update( track );
					}
					track = getTrack( i );
					manager.selectTrack( i );
					graphic.selectTrack( i );
				}
				
				// set/store the new selection 
				// and add signal if it's an empty space.
				if ( track!=null ) {
					
					// store the required io values
					ioType	= track.getSelectionType();
					ioIndex	= track.getSelectionIndex();
					
					// set the new selection
					boolean sel = track.select(time);
					
					// add segment
					if ( !sel ) track.add( time );
					
					// update GC on any change
					if ( !sel ||
						track.getSelectionType()!=ioType ||
						track.getSelectionIndex()!=ioIndex ) {
						setOutputEnabled( true );
						graphic.update( track );
						graphic.repaint();
					}
				}
				break;
		}
		
	}
	
	/* run method: 
	 * refresh the interface in the play mode
	 */
	public synchronized void run() {
		while( thread!=null ) {
			try {
				
				double x = getChrono();
				
				// wait before repaint interface
				wait();
				
				// repaint
				if ( !isFullScreen && isRunning() ) {
					timeline.setCursor( (int)x );
					graphic.setCursor( (int)x, true );
					toolbar.setPosition( x );
				}
				
			}
			catch( InterruptedException e ) {;}
		}
	}
	
	/* start all process
	 */
	public void start() {
		if ( thread==null ) {
			
			// thread
			thread = new Thread( this );
			thread.start();
			video.start( getChrono() );
			super.start();
			
			// update ui
			if ( track!=null ) {
				track.deselect();
				graphic.selectTrack( -1 );
			}
			setOutputEnabled( false );
			statusbar.setStatus( statusbar.RUN );
		}
	}
	
	/* stop all process and clear all Thread waiting status
	 */
	public void stop() {
		// BUG ??? SOME THREADS ARE NOT DELETED ? 
		// CHECK THE CURRENT THREAD WITH
		// THE Thread.currentThread() METHOD
		if ( thread!=null ) {
			
			super.stop();
			thread.interrupt();
			graphic.setCursor( (int)getChrono()-1 );
			
			// update ui
			statusbar.setStatus( statusbar.EDIT);
			if ( isFullScreen ) fswin.exit( video );
		}
		thread = null;
		video.stop();
	}
	
	/* reset the chrono value
	 */
	public void rewind() {
		setChrono( 0. );
		video.setTime( 0. );
		toolbar.setPosition( 0 );
		timeline.setCursor( 0 );
		graphic.setCursor( -1 );
	}
	
	/* load a video from file
	 */
	public void loadVideo() {
		
		// do nothing in the run mode
		if ( isRunning() ) return;
		
		FileDialog fd = new FileDialog( frame, "Select a video file" );
		
		// change status
		statusbar.setStatus( statusbar.IMPORT );
		
		// load open dialog window
		fd.setMode( FileDialog.LOAD );
		fd.setDirectory( (new File(DIR_MEDIAS)).getAbsolutePath() );
		fd.show();
		if ( fd.getDirectory()!=null && fd.getFile()!=null) {
			
			String f = fd.getDirectory()+fd.getFile();
			
			// on success
			if ( video.load( "file:"+f) ) {
				setVideo( f );
				videoPane.setText( "video : "+fd.getFile() );
				videoPane.setExpanded( true );
				videoPane.setEnabled( true );
				setModified( true );
				//video.prefetch();
			}
			// on error
			else {
				videoPane.setText( "video : loading movie \""+f+"\" failed!" );
				videoPane.setExpanded( false );
				videoPane.setEnabled( false );
			}
		}
		
		// change status (again)
		statusbar.setStatus( statusbar.EDIT );
	}
	
	/* remove the current video
	 */
	private void removeVideo() {
		if ( getVideo()!=null ) {
			video.clear();
			setVideo( "" );
			videoPane.setText( "video : " );
			videoPane.setExpanded( false );
			videoPane.setEnabled( false );
		}
	}
	
	/* display video in full the full screen window
	 * or exit the full screen mode
	 * @arg boolean, full screen value
	 */
	private void setFullScreen( boolean fsval ) {
		isFullScreen = fsval;
		if ( isFullScreen ) fswin.enter( video );
		else fswin.exit( video );
	}
	
	/* override the save method
	 */
	public void save() {
		if ( getFile()==null ) saveAs();
		else {
			setModified( false );
			super.save();
		}
	}
	
	/* open a save dialog window save the project data
	 */
	private void saveAs() {
		
		String filename	= getName()+".xml";
		FileDialog fd	= new FileDialog( frame, "Save" );
		
		// load save dialog window
		fd.setMode( FileDialog.SAVE );
		fd.setFile( filename );
		fd.setDirectory( (new File(DIR_PROJECTS)).getAbsolutePath() );
		fd.show();
		if ( fd.getDirectory() != null && fd.getFile() != null) {
			
			filename = fd.getFile();
			if ( !filename.endsWith(".xml") ) filename = fd.getFile()+".xml";
			
			setFile( fd.getDirectory() + filename );
			setName( filename );
			setModified( false );
			super.save();
		}
	}
	
	/* open a choosed project
	 */
	private void open() {
		
		FileDialog fd = new FileDialog( frame, "Open" );
		
		// load window
		fd.setMode( FileDialog.LOAD );
		fd.setDirectory( (new File(DIR_PROJECTS)).getAbsolutePath() );
		fd.setFilenameFilter( new XMLFilter() );
		fd.show();
		if ( fd.getDirectory()!=null && fd.getFile()!=null) 
			open( fd.getDirectory()+fd.getFile() );
	}
	
	/* open the passed project file
	 */
	private void open( String file ) {
		if ( (!modified && getName().equals(DEFAULT_PROJECT_NAME)) ||
			 file.equals( getName()+".xml" ) ) close();
		new Editor( winMgr, new File(file) );
	}
	
	/* close the current frame
	 */
	public void close() {
		
		Object[] options	= { "Save", "Cancel", "Don't Save" };
		String  message		= "Do you want to save the changes you made\n"+
							  "in the project \""+getName()+"\"";
		
		// display a "save before closing" alert (if needed)
		if ( modified ) {
			int result = JOptionPane.showOptionDialog( 
							frame, 
							message, 
							"",
							JOptionPane.YES_NO_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE,
							null,
							options,
							options[0]
			);
			if ( result==0 ) save(); // save
			if ( result==1 ) return; // cancel
		}
		
		// close
		frame.dispose();
	}
	
	/* change the current editor status
	 */
	private void setModified( boolean status ) {
		modified = status;
		frame.setTitle( status ? "~ "+getName(): getName() );
	}
	
	/* verify (and create) the dependency directories
	 */
	private void checkDependency() {
		
		File fproject	= new File( DIR_PROJECTS );
		File fmedia		= new File( DIR_MEDIAS );
		
		try {
			fproject.mkdirs();
			fmedia.mkdirs();
		}
		catch( Exception e ) { System.out.println(e.getMessage()); }
	}
	
	
	/* set the focus to the current frame to process the key events
	 */
	public void requestFocus() {
		if ( !frame.hasFocus() ) frame.requestFocus();
	}
}
