/*
 * (./) ExpandPane.java 05/11/07 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;


/* A extend JComponent used to easily show or hide his content.
 * The new components are only added in a Main Panel.
 */
public class ExpandPane extends JComponent implements MouseListener {

	// define the different icon status
	public static final int ICON_TYPE_CLOSE			= 0;
	public static final int ICON_TYPE_CLOSE_OVER	= 1;
	public static final int ICON_TYPE_EXPAND		= 2;
	public static final int ICON_TYPE_EXPAND_OVER	= 3;
	
	private boolean is_expanded;	// status
	private JButton button;			// the switch button
	private JPanel mainPane;		// the main container
	
	/* constructor
	 * @arg String, the panel label
	 */
	public ExpandPane( String name ) {
		
		// set button
		button = new JButton( name );
		button.setIconTextGap( 7 );
		button.setFocusable( false );
		button.setBorderPainted( false );
		button.setContentAreaFilled( false );
		button.setHorizontalAlignment( SwingConstants.LEFT );
		button.addMouseListener( this );
		button.setAlignmentX( Component.LEFT_ALIGNMENT );
		
		// set the main panel
		mainPane = new JPanel();
		mainPane.setAlignmentX( Component.LEFT_ALIGNMENT );
		
		// lay out components
		super.setLayout( new BoxLayout(this, BoxLayout.Y_AXIS) );
		super.add( button );
		super.add( mainPane );
		
		// set the default display status
		setExpanded( false );
	}
	
	/* override add method to include all new components in the main panel
	 * @arg Component, the new Component
	 * @arg Object, an object expressing layout contraints for this component
	 */
	public Component add( Component comp ) {
		return mainPane.add( comp );
	}
	
	/* override add method to include all new components in the main panel
	 * @arg Component, the new Component
	 * @arg Object, an object expressing layout contraints for this component
	 */
	public void add( Component comp, Object constraints) {
		mainPane.add( comp, constraints );
	}
	
	/* enabled or disabled panel action, if the passed value correspond to the 
	 * disabled status, all the content will be hidden
	 * @arg Boolean, the toggle panel status value
	 */
	public void setEnabled( boolean val ) {
		button.setEnabled( val );
		//is_expanded = false;
	}
	
	/* show or hide the main panel (not the require component)
	 * @arg boolean, display value
	 */
	public void setExpanded( boolean val ) {
		is_expanded = val;
		button.setIcon( val?getIcon(ICON_TYPE_EXPAND):getIcon(ICON_TYPE_CLOSE));
		mainPane.setVisible( val );
	}
	
	/* return panel status
	 */
	public boolean isExpanded() {
		return is_expanded;
	}
	
	/* set the main panel Layout
	 * find a soultion to call directly the setLayout() instead
	 * @arg LayoutManager, the require LayoutManager
	 */
	public void setLayout( LayoutManager mgr ) {
		mainPane.setLayout( mgr );
	}
	
	/* draw the require icon
	 * @arg int, the iocn type
	 * @type ImageIcon
	 */
	public ImageIcon getIcon( int type ) {
		
		int w = 11;
		int h = 11; 
		int[] xpoints = new int[3];
		int[] ypoints = new int[3];
		
		BufferedImage img = new BufferedImage(w,h,BufferedImage.TYPE_INT_ARGB);
		Graphics2D imgGC  = (Graphics2D) img.getGraphics();
		
		// define require value
		switch( type ) {
			case ICON_TYPE_CLOSE:
			case ICON_TYPE_CLOSE_OVER:
				xpoints = new int[]{ 1, w-2, 1 };
				ypoints = new int[]{ 0, h/2, h-1 };
				break;
			case ICON_TYPE_EXPAND:
			case ICON_TYPE_EXPAND_OVER:
				xpoints = new int[]{ 0, w-1, w/2 };
				ypoints = new int[]{ 1, 1, h-2 };
				break;
		}
		
		// draw arrow
		imgGC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
							   RenderingHints.VALUE_ANTIALIAS_ON);
		imgGC.setColor( (type%2==0) ? Color.DARK_GRAY : Color.BLACK );
		imgGC.fillPolygon( xpoints, ypoints , 3);
		
		// return the new image
		return new ImageIcon( img );
	}
	
	/* change or set the panel title
	 * @arg String, the text to be display
	 */
	public void setText( String s ) {
		button.setText( s );
	}
	
	/* mouse event actions
	 */
	public void mouseEntered( MouseEvent e ) {;}
	public void mouseReleased( MouseEvent e ){;}
	public void mouseClicked( MouseEvent e ) {
		if ( button.isEnabled() ) {
			setExpanded( !isExpanded() );
			button.setIcon( isExpanded() ?
							getIcon(ICON_TYPE_EXPAND):
							getIcon(ICON_TYPE_CLOSE)
						);
		}
	}
	public void mouseExited( MouseEvent e ) {
		if ( button.isEnabled() )
			button.setIcon( isExpanded() ?
							getIcon(ICON_TYPE_EXPAND):
							getIcon(ICON_TYPE_CLOSE)
						);
	}
	public void mousePressed( MouseEvent e ) {
		if ( button.isEnabled() )
			button.setIcon( isExpanded() ?
							getIcon(ICON_TYPE_EXPAND_OVER):
							getIcon(ICON_TYPE_CLOSE_OVER)
						);
	}
}
