/*
 * (./) Timeline.java 05/11/14 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import javax.swing.JComponent;
import javax.swing.event.MouseInputListener;

/* a graphics timeline component
*/
class Timeline extends JComponent {
	
	
	// reflect the project duration in pixels
	private int length = 0;
	
	// the cursor position
	private int cursor = 0;
	
	// define used colors
	public static final Color COLOR_ENABLE		= new Color( 240, 240, 240 );
	public static final Color COLOR_DISABLE		= Color.GRAY;
	public static final Color COLOR_FOREGROUND	= Color.LIGHT_GRAY;
	public static final Color COLOR_CURSOR		= Color.RED;
	
	
	
	/* constructor
	 */
	public Timeline() {;}
	
	/* set the enabled timeline zone 
	 * @arg, the timeline zone lengh
	 */
	public void setLength( int length ) {
		this.length = length;
		setPreferredSize( new Dimension(length+Editor.GRAPHIC_RIGHT_MARGIN,20) );
	}
	
	/* paint method
	 */
	public void paint( Graphics g ) {
		
		// paint only the visible graphic components area
		// > g.getClipBounds();
		
		int width		= getWidth();
		int height		= getHeight();
		Graphics2D gc2D = (Graphics2D) g;
		
		// set the graphic renderer
        gc2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
							  RenderingHints.VALUE_ANTIALIAS_ON);
        gc2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, 
							  RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		
		// draw background
		gc2D.setColor( COLOR_DISABLE );
		gc2D.fillRect( 0, 0, width, height );
		gc2D.setColor( COLOR_ENABLE );
		gc2D.fillRect( 0, 0, length, height );
		
		// draw the timeline
		gc2D.setColor( COLOR_FOREGROUND );
		gc2D.setFont( Editor.FONT_PLAIN );
		for( int x=0; x<width; x+=6 ) {
			if ( x%60==0 ) {
				gc2D.drawLine( x, 0, x, height-17 );
				gc2D.drawString( Toolbar.TimeFormat.format(x), x-2, height-7 );
			}
			gc2D.drawLine( x, height-5, x, height );
		}
		
		// draw the cursor
		gc2D.setColor( COLOR_CURSOR );
		gc2D.drawLine( cursor, 0, cursor, height );
		gc2D.fillPolygon( new int[]{ cursor-4, cursor+4, cursor}, 
						  new int[]{ 0, 0, 6 }, 
						  3 );
		
	}
	
	/* set the cursor position
	 * @arg int, the cursor x coordonate
	 */
	public void setCursor( int x ) {
		cursor = Math.max( Math.min(x, length), 0 );
		repaint();
	}
	
	/* set the mouse listener
	 */
	public void setMouseListener( MouseInputListener listener ) {
		addMouseListener( listener );
		addMouseMotionListener( listener );
	}
}
