/*
 * (./) Toolbar.java 05/12/27 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.*;
import javax.swing.text.NumberFormatter;
import javax.swing.text.DefaultFormatterFactory;


/* a toolbar panel containing the zoom field, the position field, the duration 
 * field, (etc.)
 */
public class Toolbar extends JPanel {
	
	
	// text field for data display
	private JFormattedTextField position;
	private JFormattedTextField duration;
	private JFormattedTextField start;
	private JFormattedTextField stop;
	private JFormattedTextField zoom;
	private JComboBox size;
	
	// define the action commands
	public static final String NEW_POSITION_VALUE	= "new position value";
	public static final String NEW_DURATION_VALUE	= "new duration value";
	public static final String NEW_START_VALUE		= "new start value";
	public static final String NEW_STOP_VALUE		= "new stop value";
	public static final String NEW_ZOOM_VALUE		= "new zoom value";
	public static final String NEW_SIZE_VALUE		= "new graphic height value";
	
	// labels
	JLabel positionlabel = new JLabel( "Position", SwingConstants.RIGHT );
	JLabel durationlabel = new JLabel( "Duration", SwingConstants.RIGHT );
	JLabel startlabel	 = new JLabel( "Start signal at", SwingConstants.RIGHT );
	JLabel stoplabel	 = new JLabel( "Stop signal at", SwingConstants.RIGHT );
	JLabel zoomlabel	 = new JLabel( "Zoom", SwingConstants.RIGHT );
	JLabel sizelabel	 = new JLabel( "Graphics heigth", SwingConstants.RIGHT );
	
	/* init components
	 */
	public Toolbar() {
		
		// set up percent number format
		NumberFormat pdf = NumberFormat.getPercentInstance();
        pdf.setMinimumFractionDigits(2);
		
		// set up labels
		positionlabel.setFont( Editor.FONT_PLAIN );
		durationlabel.setFont( Editor.FONT_PLAIN );
		startlabel.setFont( Editor.FONT_PLAIN );
		stoplabel.setFont( Editor.FONT_PLAIN );
		zoomlabel.setFont( Editor.FONT_PLAIN );
		sizelabel.setFont( Editor.FONT_PLAIN );
		
		// create text field and set them up
		position = new JFormattedTextField(
						new DefaultFormatterFactory( new TimeFormat() )
					);
		position.setFont( Editor.FONT_PLAIN );
		position.setEditable( false );
		position.setColumns( 10 );
		
		duration = new JFormattedTextField(
						new DefaultFormatterFactory( new TimeFormat() )
					);
		duration.setFont( Editor.FONT_PLAIN );
		duration.setColumns( 10 );
		
		start = new JFormattedTextField(
					new DefaultFormatterFactory( new TimeFormat() )
				);
		start.setFont( Editor.FONT_PLAIN );
		start.setColumns( 10 );
			
		stop = new JFormattedTextField(
					new DefaultFormatterFactory( new TimeFormat() )
				);
		stop.setFont( Editor.FONT_PLAIN );
		stop.setColumns( 10 );
		
		zoom = new JFormattedTextField(
					 new DefaultFormatterFactory(
							new NumberFormatter( pdf ),
							new NumberFormatter( pdf ),
							new PercentFormat()
					)
				);
		zoom.setFont( Editor.FONT_PLAIN );
		zoom.setColumns( 10 );
		zoomlabel.setEnabled( false );
		zoom.setEnabled( false );
		
		// create combobox
		size = new JComboBox();
		size.setFont( Editor.FONT_PLAIN );
		size.setBounds( 0, 0, 100, 10 );
		size.setPreferredSize( new Dimension(0, 10) );
		size.addItem( new String("x-small") );
		size.addItem( new String("small") );
		size.addItem( new String("medium") );
		size.addItem( new String("large") );
		size.addItem( new String("x-large") );
		size.setSelectedIndex( 2 );
		
		// layout components
		setBorder( BorderFactory.createEmptyBorder(3, 7, 0, 7) );
		setLayout( new GridLayout( 0, 8, 7, 3 ) );
		add( positionlabel );
		add( position );
		add( startlabel );
		add( start );
		add( zoomlabel );
		add( zoom );
		add( sizelabel );
		add( size );
		add( durationlabel );
		add( duration );
		add( stoplabel );
		add( stop );
		
		setOutputFieldsEnabled( false );
	}
	
	/* add the action listener to the text fields
	 */
	public void setActionListener( ActionListener listener ) {
		position.setActionCommand( NEW_POSITION_VALUE );
		position.addActionListener( listener );
		duration.setActionCommand( NEW_DURATION_VALUE );
		duration.addActionListener( listener );
		start.setActionCommand( NEW_START_VALUE );
		start.addActionListener( listener );
		stop.setActionCommand( NEW_STOP_VALUE );
		stop.addActionListener( listener );
		zoom.setActionCommand( NEW_ZOOM_VALUE );
		zoom.addActionListener( listener );
		size.setActionCommand( NEW_SIZE_VALUE );
		size.addActionListener( listener );
	}
	
	/* get or set the position field
	 */
	public void setPosition( double d ) { position.setValue( new Double(d) ); }
	public double getPosition() {
		return ((Number)position.getValue()).doubleValue();
	}
	
	/* get or set the duration field
	 */
	public void setDuration( double d ) { duration.setValue( new Double(d) ); }
	public double getDuration() {
		return ((Number)duration.getValue()).doubleValue();
	}
	
	/* get or set the start field
	 */
	public void setStart( double d ) { start.setValue( new Double(d) ); }
	public double getStart() { 
		return ((Number)start.getValue()).doubleValue(); 
	}
	
	/* get or set the end field
	 */
	public void setStop( double d ) { stop.setValue( new Double(d) ); }
	public double getStop() {
		return ((Number)stop.getValue()).doubleValue();
	}
	
	/* get or set the zoom field
	 */
	public void setZoom( double d ) { zoom.setValue( new Double(d) ); }
	public double getZoom() {
		return ((Number)zoom.getValue()).doubleValue();
	}
	
	/* get the current size value
	 */
	public int getGraphicSize() {
		int[] values = { 
			TrackGraphic.SIZE_XSMALL,
			TrackGraphic.SIZE_SMALL, 
			TrackGraphic.SIZE_MEDIUM,
			TrackGraphic.SIZE_LARGE,
			TrackGraphic.SIZE_XLARGE
		};
		return values[ size.getSelectedIndex() ];
	}
	
	/* set both start end stop fields enable
	 */
	public void setOutputFieldsEnabled( boolean val ) {
		startlabel.setEnabled( val );
		stoplabel.setEnabled( val );
		start.setEnabled( val );
		stop.setEnabled( val );
	}
	
	/* the time field editor
	 * set the time value to a formatted string hh:mm::ss
	 */
	static class TimeFormat extends NumberFormatter {
		
		
		public static final long	ONE_SECOND	= 1000000000L;
		public static final double	NANO_TO_SEC	= 1.0E-9;
		
		
		// convert the value to string
		public String valueToString( Object o ) throws ParseException {
			Number n = (Number)o;
			if ( n!=null ) return format( n.doubleValue() );
			return format( 0l );
		}
		
		// format the passed number
		public static String format( int i )	{ return format(i*ONE_SECOND); }
		public static String format( double d ) { return format((long)d*ONE_SECOND); }
		public static String format( long n )	{ // n = nanoseconds
			
			int s  = (int)(n/ONE_SECOND);
			int h  = s/3600;
			int m  = ( s - (h*3600)) / 60;
			s = s - (h*3600) - (m*60);
			n = (long) ((n % ONE_SECOND)/(ONE_SECOND/100));
			
			return new String( (h/10) +""+ (h%10) +":"+ 
							   (m/10) + (m%10) +":"+ 
							   (s/10) + (s%10) 
							   //+ "." + (n/10) + n
							   );
		}
		
		// convert the string to the value
		public Object stringToValue( String str ) throws ParseException {
			
			int h=0;
			int m=0;
			int	s=0;
			long n;
			
			String[] split = str.split(":");
			
			try {
				if ( split.length>=1 ) h = Integer.parseInt( split[0] );
				if ( split.length>=2 ) m = Integer.parseInt( split[1] );
				if ( split.length>=3 ) s = Integer.parseInt( split[2] );
			}
			catch( NumberFormatException e ) {;}
			
			s = (h*3600) + (m*60) + s;
			n = s*ONE_SECOND;
			
			return new Double( n*NANO_TO_SEC );
		}
	}
	
	/* the percent editor class
	 * set the passed number as a % number
	 */
	class PercentFormat extends NumberFormatter {
		
		// set format
		public PercentFormat() {
			NumberFormat nf = NumberFormat.getNumberInstance();
			nf.setMinimumFractionDigits(2);
			setFormat( nf );
		}
		
		// convert the value to string 
		public String valueToString( Object o ) throws ParseException {
			Number n = (Number)o;
			if ( n!=null ) n = new Double( n.doubleValue()*100.0 );
			return super.valueToString(n);
		}
		
		// convert the string to the value
		public Object stringToValue( String s ) throws ParseException {
			Number n = (Number)super.stringToValue(s);
			if ( n!=null ) n = new Double( n.doubleValue()/100.0 );
			return n;
		}
	}
}

