/*
 * (./) TrackManager.java 05/11/08 by cs for "MMM" project
 * (cc) some right reserved
 */

package mmm.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;

import mmm.Project;
import mmm.ui.manager.*;

/* a single list to manage the project's tracks: add, rename, select, 
 * hide/show, mute, etc...
 */
public class TrackManager extends JScrollPane {
	
	// define colors
	public final static Color BACKGROUND_COLOR		= Color.WHITE;
	public final static Color ALTERNATE_BKG_COLOR	= new Color(247, 247, 247);
	
	// list var
	private JTable table					= null;
	private DefaultTableModel tableModel	= null;


	/* create a new list
	 */
	public TrackManager() {
		
		TableColumn tc0, tc1, tc2, tc3, tc4;
		
		// customize the new table : disallow edit and force indexing
		// on the first column
		table = new JTable() {
			public boolean isCellEditable( int r, int c ) { 
				return c!=0; 
			}
			public Object getValueAt( int r, int c ) {
				return c==0 ? new Integer(r+1) : super.getValueAt(r, c);
			}
			public Class getColumnClass( int c ) {
				return getValueAt(0, c).getClass();
			}
		};
		table.setTableHeader( null );
		table.setGridColor( BACKGROUND_COLOR );
		table.setBackground( BACKGROUND_COLOR );
		table.setFont( Editor.FONT_PLAIN );
		table.setRowMargin( 0 );
		table.setAutoCreateColumnsFromModel( false );
		table.setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );
		
		// index
		tc0 = new TableColumn( 0, 20, new IndexCellRenderer(), null);
		tc0.setMinWidth( 20 );
		tc0.setMaxWidth( 20 );
		
		// set enable 'button'
		tc1 = new TableColumn( 1, 20 );
		tc1.setCellRenderer( new EnableCellRenderer() ); 
		tc1.setCellEditor( new BooleanCellEditor() );
		tc1.setMinWidth( 20 );
		tc1.setMaxWidth( 20 );
		
		// set display 'button'
		tc2 = new TableColumn( 2, 20 );
		tc2.setCellRenderer( new DisplayCellRenderer() ); 
		tc2.setCellEditor( new BooleanCellEditor() );
		tc2.setMinWidth( 20 );
		tc2.setMaxWidth( 20 );
		
		// color
		tc3 = new TableColumn( 3, 16 );
		tc3.setCellRenderer( new ColorCellRenderer() );
		tc3.setCellEditor( new ColorCellEditor(table.getSelectionBackground()) );
		tc3.setMinWidth( 16 );
		tc3.setMaxWidth( 16 );
		
		// name
		tc4 = new TableColumn(4, 100);
		tc4.setCellRenderer( new StringCellRenderer() );
		tc4.setCellEditor( new StringCellEditor() );
		
		// set the model
		tableModel = (DefaultTableModel) table.getModel();
		
		// add columns
		table.addColumn( tc0 );
		table.addColumn( tc1 );
		table.addColumn( tc2 );
		table.addColumn( tc3 );
		table.addColumn( tc4 );
		tableModel.addColumn( tc0 );
		tableModel.addColumn( tc1 );
		tableModel.addColumn( tc2 );
		tableModel.addColumn( tc3 );
		tableModel.addColumn( tc4 );
		
		// set the scroll pane
		setMinimumSize( new Dimension(100, 30) );
		setPreferredSize( new Dimension(150, 30) );
		setViewportView( table );
	}
	
	/* append a new item with the default values
	 */
	public void add() {
		add( new Object[] {
			null, // auto indexing items
			new Boolean(true), 
			new Boolean(true), 
			Project.DEFAULT_TRACK_COLOR, 
			Project.DEFAULT_TRACK_NAME + "." + (table.getRowCount()+1)
		});
	}
	
	/* append a new item with the passed values
	 * @arg Object[], data values
	 */
	public void add( Object[] data ) {
		tableModel.addRow( data );
	}
	
	/* set up all items
	 * @arg Object[][], items values
	 */
	public void add( Object[][] data ) {
		for(int i=0; i<data.length; i++) add( data[i] );
	}
	
	/* remove selected item(s)
	 */
	public void remove() {
		
		int sel		= 0;
		int rowcnt	= table.getRowCount();
		int[] items = table.getSelectedRows();
		
		// try to select the next or the previous row first
		if ( items.length==1 ) { 
			if ( rowcnt==1 ) table.clearSelection();
			else {
				sel = items[0]<rowcnt-1 ? items[0]+1 : items[0]-1;
				selectTrack( sel );
			}
		}
		else { 
			if ( rowcnt==items.length ) table.clearSelection();
			else {
				sel = items[items.length-1]<rowcnt-1 ? 
					  items[items.length-1]+1 : items[items.length-1]-1;
				selectTrack( sel );
			}
		}
		
		// remove
		for(int i=(items.length-1); i>=0; i--) tableModel.removeRow( items[i] );
	}
	
	/* move down selected item(s)
	 */
	public void moveDown() {
		int[] items = table.getSelectedRows();
		if ( items[0]!=0 ) moveBy( items, -1 );
	}
	
	/* move up selected item(s)
	 */
	public void moveUp() {
		int[] items = table.getSelectedRows();
		if ( items[items.length-1]!=table.getRowCount()-1 ) 
			moveBy( items, 1 );
	}
	
	/* move selected item to a specifique position
	 * @arg int[], the items index
	 * @arg int, the target position
	 */
	private void moveTo( int[] items, int to ) {
		tableModel.moveRow( items[0], items[items.length-1], to);
	}
	
	/* move selected items one by one by a specifique range
	 * @arg int[], the items index
	 * @arg int, the row range
	 */
	private void moveBy( int[] items, int by ) {
		deselectAll();
		for( int i=0; i<items.length; i++ ) {
			// fix bug : if by is positive, the "moveBy" method needs to perform  
			// the action from the end to the beginning.
			int row = (by<0) ? items[i] : items[items.length-1-i];
			tableModel.moveRow( row, row, row+by );
			table.addRowSelectionInterval( row+by, row+by );
		}
	}
	
	/* insert a new item at teh selected index
	 */
	public void insert() {
		
		int[] items  = table.getSelectedRows();
		if ( items.length==0 ) return; // no selection
		
		tableModel.insertRow( items[0], new Object[] {
			null, // auto indexing items
			new Boolean(true), 
			new Boolean(true), 
			Project.DEFAULT_TRACK_COLOR, 
			Project.DEFAULT_TRACK_NAME + "." + (items[0]+1)
		});
	}
	
	/* set tableModel listener
	 * @arg TableModelListener, the table listener
	 */
	public void setTableModelListener( TableModelListener listener ) {
		tableModel.addTableModelListener( listener );
	}
	
	/* select a specific track
	 * @arg int, track index
	 */
	public void selectTrack( int i ) {
		table.setRowSelectionInterval( i, i );
	}
	
	/* deselects all selected items
	 */
	public void deselectAll() {
		table.clearSelection();
	}
	
	/* duplicate selected tracks
	 */
	public void duplicate() {
		
		int[] items = table.getSelectedRows();
		if ( items.length==0 ) return; // no selection
		
		for( int i=(items.length-1); i>=0; i-- )  {
			
			int at		= items[i]+1;
			Object[] o	= new Object[] {
								null, // auto indexing items
								(Boolean) tableModel.getValueAt(items[i], 1),
								(Boolean) tableModel.getValueAt(items[i], 2),
								(Color) tableModel.getValueAt(items[i], 3),
								(String) tableModel.getValueAt(items[i], 4) + " copie",
							};
			
			if ( at==items.length ) tableModel.addRow( o );
			else tableModel.insertRow( at, o );
		}
	}
}
